import gnu.xquery.lang.XQuery;
import java.util.*;
import gnu.lists.*;
import java.io.PrintStream;

public class CountElements extends FilterConsumer
{
  CountElements()
  {
    super(VoidConsumer.getInstance());
  }

  List elementNames = new ArrayList();
  int numAttributes = 0;
  int numInts = 0;
  int numObjects = 0;

  public void beginGroup(String typeName, Object type)
  {
    elementNames.add(typeName);
    super.beginGroup(typeName, type);
  }

  public void beginAttribute(String attrName, Object attrType)
  {
    numAttributes++;
    super.beginAttribute(attrName, attrType);
  }

  public void writeInt(int v)
  {
    numInts++;
    super.writeInt(v);
  }

  public void writeObject(Object v)
  {
    numObjects++;
    super.writeObject(v);
  }

  void dump (PrintStream out)
  {
    Collections.sort(elementNames);
    int total = 0;
    ListIterator it = elementNames.listIterator();
    String previous = null;
    int count = 0;
    for (;;)
      {
	boolean done = ! it.hasNext();
	String cur = done ? "" : (String) it.next();
	if (previous != null && ! previous.equals(cur))
	  {
	    out.println("<" + previous + "> - " + count + " times");
	    count = 0;
	  }
	if (done)
	  break;
	previous = cur;
	count++;
	total++;
      }
    out.println("TOTAL: " + total);
    if (numAttributes > 0)
      out.println("Attributes: " + numAttributes);
    if (numInts > 0)
      out.println("ints: " + numInts);
    if (numObjects > 0)
      out.println("Objects: " + numObjects);
  }

  public static void main(String[] args) throws Throwable
  {
    XQuery xq = new XQuery();
    CountElements counter = new CountElements();
    for (int i = 0;  i < args.length;  i++)
      {
	String exp = args[i];
	xq.eval(exp, counter);
      }
    counter.dump(System.out);
  }
}
